/**
  *  \file tests/movieread.h
  *
  *  (c) 2001-2011 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_TESTS_MOVIEREAD_H
#define PCC_V2_TESTS_MOVIEREAD_H

#include <cstddef>
#include "cpluslib/object.h"
#include "cpluslib/buffer.h"
#include "cpluslib/string.h"

class Stream;
class MovieReader : public Object {
    Buffer<char> data;
    std::size_t  ip;

    unsigned char getByte(std::size_t ip);
    unsigned int  getWord(std::size_t ip);
    unsigned char getByte() { return getByte(ip++); }
    unsigned int  getWord() { return getWord((ip += 2) - 2); }
 public:
    MovieReader();
    ~MovieReader();

    void loadFromStream(Stream& s);

    int read();

    int getPlanetX(int id) { return getWord(6 + 4*id); }
    int getPlanetY(int id) { return getWord(8 + 4*id); }

    void        setPosition(std::size_t ip) { this->ip = ip; }
    std::size_t getPosition() const { return this->ip; }

    const Buffer<char>& getData() const { return data; }

    virtual void error(string_t s);
    virtual void newFile();

    /* callbacks */
    virtual int newTurn() = 0;
    virtual int newBlock(int type) = 0;
    virtual void changeMinefield(int id, int x, int y, int radius, int ot) = 0;
    virtual void changeMinefield(int id, int radius) = 0;
    virtual void changePlanet(int id, int owner, bool base) = 0;
    virtual void changeShip(int id, int what, int hull, int oe,
                            int beam, int torp, int damage, int x, int y) = 0;

    enum {
        si_Hull        = 1,
        si_OwnerEngine = 2,
        si_Beam        = 4,
        si_Torp        = 8,
        si_Damage      = 16,
        si_X           = 32,
        si_Y           = 64
    };

    enum {
        bt_NewTurn   = 0,
        bt_Planet    = 1,
        bt_Ship      = 2,
        bt_Minefield = 3
    };

    enum { START_IP = 1000*2 + 10 + 682 };
};

class MovieSimpleReader : public MovieReader {
    int turn_limit;
    int turn;
 public:
    MovieSimpleReader(int turn_limit = -1);

    int newTurn();
    int newBlock(int type);
    void changeMinefield(int id, int x, int y, int radius, int ot);
    void changeMinefield(int id, int radius);
    void changePlanet(int id, int owner, bool base);
    void changeShip(int id, int what, int hull, int oe,
                    int beam, int torp, int damage, int x, int y);

    void setTurn(int turn) { this->turn = turn; }
    int  getTurn() const { return this->turn; }
};

#endif
